package de.pereira.aqfinder;

import java.util.List;

import android.app.Activity;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Toast;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;

import de.android1.overlaymanager.ManagedOverlay;
import de.android1.overlaymanager.OverlayManager;
import de.pereira.aqfinder.listener.MyLazyLoadCallback;
import de.pereira.aqfinder.listener.MyOnOverlayGestureListener;
import de.pereira.aqfinder.util.MyLocationProvider;
import de.pereira.aqfinder.util.MyLocationProvider.LocationResult;

public class AQFinder extends MapActivity {
	
    public static final String SIR_URL = "http://giv-genesis.uni-muenster.de:8080/SIR/sir";
    
	private static final int SEARCH_TEXT_MENU_ITEM = 0;
	private MapView mapView; 
	private List<Overlay> mapOverlays;
	private Drawable drawable;
	private OverlayManager overlayManager;
	private ManagedOverlay managedOverlay;
	private MapController mapController;
	private GeoPoint center = null;
	private int zoomlevel = 10;
	
	/**
	 * callback, when a location is found
	 */
	private LocationResult locationResult = new LocationResult(){

		@Override
		public void gotLocation(Location location) {
			if (location != null) {
				center = new GeoPoint((int) (location.getLatitude() * 1E6), (int) (location.getLongitude() * 1E6));
				mapController.animateTo(center);
				managedOverlay.invokeLazyLoad(500);
			}
		}
	};
	
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        // get the search term
        startSearchTermActivity();
        
        // get the item and adjust the centre to bottom mid
        drawable = this.getResources().getDrawable(R.drawable.antenna_small);
        drawable.setBounds(0 - drawable.getIntrinsicWidth()/2, -drawable.getIntrinsicHeight(), drawable.getIntrinsicWidth()/2, 0); 
        
       // initialize the map
        setContentView(R.layout.main);
        mapView = (MapView) findViewById(R.id.mapview);
        mapView.setBuiltInZoomControls(true);
        mapView.setSatellite(false);
        mapView.setClickable(true);
        
        mapController = mapView.getController();
        
        // get the location
        MyLocationProvider locProvider = new MyLocationProvider();
        if (!locProvider.getLocation(this, locationResult)) {
        	Toast.makeText(getApplicationContext(), "No location provider found. Please enable Wi-Fi or GPS module.", 10000).show();
		}
        
        // create an overlay for the markers
        overlayManager = new OverlayManager(this, mapView);
        managedOverlay = overlayManager.createOverlay(drawable);
        managedOverlay.setOnOverlayGestureListener(new MyOnOverlayGestureListener(this));
        
        mapOverlays = mapView.getOverlays();
        mapOverlays.add(managedOverlay);
        
      
        // in case no location is found. Set the center to a default.
        if (center == null) {
        	String coordinates[] = {"50.819818262156545", "9.84375"};
            double lat = Double.parseDouble(coordinates[0]);
            double lng = Double.parseDouble(coordinates[1]);
     
            center = new GeoPoint((int) (lat * 1E6), (int) (lng * 1E6));
            mapController.setCenter(center);
		}
        mapController.setZoom(zoomlevel);
    }

    
    // init the menu button
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		menu.add(0, SEARCH_TEXT_MENU_ITEM, 0, "Change search text");
		return true;
	}

	
	// triggered when a menu button has been slected
	@Override
	public boolean onMenuItemSelected(int featureId, MenuItem item) {
		switch (featureId) {
		case SEARCH_TEXT_MENU_ITEM:
			startSearchTermActivity();
			return true;

		default:
			return super.onMenuItemSelected(featureId, item);
		}
	}

	
	private void startSearchTermActivity() {
		Intent newIntent = new Intent(this, ChooseSearchString.class);
        startActivityForResult(newIntent, 0);
	}
	
	
	// called to process the result of sub activities
	/* (non-Javadoc)
	 * @see android.app.Activity#onActivityResult(int, int, android.content.Intent)
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (resultCode == Activity.RESULT_OK) {
			String searchTerm = data.getExtras().getString("searchTerm");
			Log.d("AQFinder", "Search term: "+searchTerm);
			
			// add a callback listener, that is called on every pan and zoom
			managedOverlay.setLazyLoadCallback(new MyLazyLoadCallback(searchTerm));
			managedOverlay.invokeLazyLoad(500);
		}
		
	}

	
	// mandatory implementation
	@Override
	protected boolean isRouteDisplayed() {
		return false;
	}
}