package de.pereira.aqfinder;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;

import org.achartengine.ChartFactory;
import org.achartengine.chart.PointStyle;
import org.achartengine.model.TimeSeries;
import org.achartengine.model.XYMultipleSeriesDataset;
import org.achartengine.renderer.XYMultipleSeriesRenderer;
import org.achartengine.renderer.XYSeriesRenderer;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;

import android.app.Activity;
import android.app.DatePickerDialog;
import android.app.DatePickerDialog.OnDateSetListener;
import android.app.Dialog;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.Paint.Align;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;
import de.pereira.aqfinder.model.OfferingValues;
import de.pereira.aqfinder.model.SensorItem;
import de.pereira.aqfinder.util.DateUtils;
import de.pereira.aqfinder.util.swe.ObservationRequest;

public class ChooseData extends Activity {
	
	private TextView startDateView;
	private TextView endDateView;
    
    private int from_year, from_month, from_day, to_year, to_month, to_day; 
    private OnDateSetListener from_dateListener,to_dateListener;

    static final int DATE_PICKER_FROM = 0;
    static final int DATE_PICKER_TO = 1;

    
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setContentView(R.layout.choosedata);
		
		initializeDatePickers();
		
		LinearLayout layout = (LinearLayout) findViewById(R.id.choosedata);
		LinearLayout.LayoutParams p = new LinearLayout.LayoutParams(
                LinearLayout.LayoutParams.MATCH_PARENT,
                LinearLayout.LayoutParams.WRAP_CONTENT
        );
		
		final Bundle b = getIntent().getExtras();
		final SensorItem sensorItem = (SensorItem) b.getSerializable("sensorItem");
		
		
		// for each offering add a button
		for (int i = 0; i < sensorItem.getOfferings().length; i++) {
			
			final int j = i;
			final String offering = sensorItem.getOfferings()[i].getId();
			final String observedProperty = sensorItem.getOfferings()[i].getObservedProperty();
			final String srsName = sensorItem.getOfferings()[i].getSrsName();
			
			final Button button = new Button(this);
			button.setText(sensorItem.getOfferings()[i].getName());
			button.setOnClickListener(new View.OnClickListener() {
				
				@Override
				public void onClick(View v) {
					Log.d("ChooseData", button.getText()+" selected");
					Date chosenStart = DateUtils.parseFromDatePicker(startDateView.getText().toString());
					Date chosenEnd = DateUtils.parseFromDatePicker(endDateView.getText().toString());
					SimpleDateFormat formatter = new SimpleDateFormat("dd-MMM-yyyy");
					
					if (chosenStart.before(sensorItem.getOfferings()[j].getStart())) {
						Toast.makeText(getApplicationContext(), "Earliest available data start on "+formatter.format(sensorItem.getOfferings()[j].getStart()), 14000).show();
						return;
					} else if (chosenEnd.after(sensorItem.getOfferings()[j].getEnd())) {
						Toast.makeText(getApplicationContext(), "Latest available data is from "+formatter.format(sensorItem.getOfferings()[j].getEnd()), 14000).show();
						return;
					}
					
					// make an getObservationRequest with the given parameters
					try {
						HttpResponse response = ObservationRequest.makeGetObservationRequest(sensorItem.getServiceURL(), sensorItem.getId(), offering, observedProperty, srsName, chosenStart, chosenEnd);
						OfferingValues offeringValues = ObservationRequest.parseGetObservationRequest(response);
						LinkedHashMap<String, String> data = offeringValues.getDate_values();
						
						if (data == null) {
							Log.e("ChooseData", "Error retrieving observation measurements.");
							Toast.makeText(getApplicationContext(), "Error retrieving observation measurements.", 7000).show();
							return;
						}else if (data.isEmpty()) {
							Log.e("ChooseData", "no data found for given time period");
							Toast.makeText(getApplicationContext(), "No data found for given time period.", 7000).show();
							return;
						}
						
						// create a timeseries for the chart by iterating through the date-value pairs
						TimeSeries series = new TimeSeries(button.getText().toString() + " ["+offeringValues.getUom()+"]");
						for (Iterator<String> iterator2 = data.keySet().iterator(); iterator2.hasNext();) {
							String x_data = (String) iterator2.next();
							series.add(DateUtils.parseFromXML(x_data), Double.parseDouble(data.get(x_data)));
						}
						
						XYMultipleSeriesDataset dataset = new XYMultipleSeriesDataset();
						dataset.addSeries(series);
						
						int[] colors = new int[] { Color.GREEN};
						PointStyle[] styles = new PointStyle[] {PointStyle.POINT};
						XYMultipleSeriesRenderer renderer = buildRenderer(colors, styles);
						renderer.setXLabels(4);
						renderer.setYLabels(7);
						renderer.setXLabelsAlign(Align.CENTER);
						renderer.setYLabelsAlign(Align.CENTER);
						renderer.setZoomButtonsVisible(true);
						
						Intent myIntent = ChartFactory.getTimeChartIntent(ChooseData.this, dataset, renderer, "dd-MM kk:mm");
						startActivity(myIntent);
					} catch (ClientProtocolException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (NumberFormatException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			});
			layout.addView(button, p);
		}
		
		// initialize the dateListener
		from_dateListener = new OnDateSetListener() {
			
			@Override
			public void onDateSet(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
				SimpleDateFormat formatter = new SimpleDateFormat("MMM-dd-yyyy");
				startDateView.setText(formatter.format(new Date(year-1900, monthOfYear, dayOfMonth)));
			}
		};
		to_dateListener = new OnDateSetListener() {
			
			@Override
			public void onDateSet(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
				SimpleDateFormat formatter = new SimpleDateFormat("MMM-dd-yyyy");
				endDateView.setText(formatter.format(new Date(year-1900, monthOfYear, dayOfMonth)));
			}
		};
    }
	
	
	@Override
	protected Dialog onCreateDialog(int id) {

	    switch(id){
	        case DATE_PICKER_FROM:
	                return new DatePickerDialog(this, from_dateListener, from_year+1900, from_month, from_day); 
	            case DATE_PICKER_TO:
	                return new DatePickerDialog(this, to_dateListener, to_year+1900, to_month, to_day);
	    }
	        return null;
	}

    
	private void initializeDatePickers() {
		Calendar c = Calendar.getInstance();
	    from_year = to_year = c.get(Calendar.YEAR) - 1900;
	    from_month = to_month = c.get(Calendar.MONTH);
	    from_day = to_day = c.get(Calendar.DAY_OF_MONTH);
	    
		SimpleDateFormat formatter = new SimpleDateFormat("MMM-dd-yyyy");
		Date date = new Date(from_year, from_month, from_day);
		
		startDateView = (TextView) findViewById(R.id.startDateDisplay);
		startDateView.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				showDialog(DATE_PICKER_FROM);
			}
		});
		startDateView.setText(formatter.format(date));
		
		endDateView = (TextView) findViewById(R.id.endDateDisplay);
		endDateView.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				showDialog(DATE_PICKER_TO);
				
			}
		});
		endDateView.setText(formatter.format(date));
	}
	
	
    protected XYMultipleSeriesRenderer buildRenderer(int[] colors,
			PointStyle[] styles) {
		XYMultipleSeriesRenderer renderer = new XYMultipleSeriesRenderer();
		setRenderer(renderer, colors, styles);
		return renderer;
	}

	protected void setRenderer(XYMultipleSeriesRenderer renderer, int[] colors, PointStyle[] styles) {
		renderer.setAxisTitleTextSize(16);
		renderer.setChartTitleTextSize(20);
		renderer.setLabelsTextSize(15);
		renderer.setLegendTextSize(15);
		renderer.setPointSize(5f);
		renderer.setMargins(new int[] { 20, 30, 15, 20 });
		int length = colors.length;
		for (int i = 0; i < length; i++) {
			XYSeriesRenderer r = new XYSeriesRenderer();
			r.setColor(colors[i]);
			r.setPointStyle(styles[i]);
			renderer.addSeriesRenderer(r);
		}
	}
}
