package de.pereira.aqfinder.util.swe;

import java.io.IOException;
import java.util.ArrayList;

import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import android.util.Log;
import android.util.Xml;
import de.pereira.aqfinder.model.Offering;
import de.pereira.aqfinder.model.SensorItem;
import de.pereira.aqfinder.util.DateUtils;
import de.pereira.aqfinder.util.NetworkUtils;

public class CapabilitiesRequest {
	
	static final String SRS_NAME = "urn:ogc:def:crs:EPSG:4326";
	

	public static HttpResponse makeGetCapabilitiesRequest(String server) throws ClientProtocolException, IOException {
		Log.d("CapabilitiesRequest", "Starting getCapabilities request");
		Log.d("CapabilitiesRequest", "Server: "+server);
		
		String request ="<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
				"<GetCapabilities xmlns=\"http://www.opengis.net/sos/1.0\"" +
					" xmlns:ows=\"http://www.opengis.net/ows/1.1\"" +
					" xmlns:ogc=\"http://www.opengis.net/ogc\"" +
					" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"" +
					" xsi:schemaLocation=\"http://www.opengis.net/sos/1.0 http://schemas.opengis.net/sos/1.0.0/sosGetCapabilities.xsd\"" +
					" service=\"SOS\">" +
				"" +
				"<ows:AcceptVersions>" +
					"<ows:Version>1.0.0</ows:Version>" +
				"</ows:AcceptVersions>" +
				"" +
				"<ows:Sections>" +
//					"<ows:Section>OperationsMetadata</ows:Section>" +
//					"<ows:Section>ServiceIdentification</ows:Section>" +
//					"<ows:Section>ServiceProvider</ows:Section>" +
//					"<ows:Section>Filter_Capabilities</ows:Section>" +
					"<ows:Section>Contents</ows:Section>" +
				"</ows:Sections>" +
				"" +
				"</GetCapabilities>";
		
		Log.d("CapabilitiesRequest", "getCapabilities request: "+request);
		
		HttpResponse response = NetworkUtils.execute(server, request);
		Log.d("CapabilitiesRequest", "getCapabilities request completed");

		return response; 
	}
	
	
	public static SensorItem parseGetCapabilitiesResponse(HttpResponse response, String procedure){
		ArrayList<Offering> offeringList = new ArrayList<Offering>();
		
		XmlPullParser parser = Xml.newPullParser();
		try {
			parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, true);
			parser.setInput(response.getEntity().getContent(), null);
			
			int eventType = parser.getEventType();
			Offering offering = null;
			boolean sensorIDfound = false;
			
			while (eventType != XmlPullParser.END_DOCUMENT) {
				String name = null;
				switch (eventType) {
				case XmlPullParser.START_TAG:
						name = parser.getName();
					
					if (name.equalsIgnoreCase("ObservationOffering")) {
						offering = new Offering();
						offering.setId(parser.getAttributeValue(0));
						sensorIDfound = false;
					} else if (offering != null) {
						if (name.equalsIgnoreCase("name")) {
							offering.setName(parser.nextText());
						} else if (name.equalsIgnoreCase("beginPosition")) {
							offering.setStart(DateUtils.parseFromXML(parser.nextText()));
						} else if (name.equalsIgnoreCase("endPosition")) {
							offering.setEnd(DateUtils.parseFromXML(parser.nextText()));
						} else if (name.equalsIgnoreCase("Envelope")) {
							offering.setSrsName(parser.getAttributeValue(0));
						} else if (!sensorIDfound && name.equalsIgnoreCase("procedure")) {
							String sensorID = parser.getAttributeValue(0);
							if (sensorID.equalsIgnoreCase(procedure)) {
								sensorIDfound = true;
							}
						} else if (name.equalsIgnoreCase("observedProperty")) {
							String observedPropertyValue = parser.getAttributeValue(0);
							if (!observedPropertyValue.startsWith("urn:x-")) {
								offering.setObservedProperty(observedPropertyValue); 
							}
						}
					}
					break;
					
				case XmlPullParser.END_TAG:
					name = parser.getName();
					if (name.equalsIgnoreCase("ObservationOffering")) {
						if (sensorIDfound && offering!=null && offering.isValid()) {
							offeringList.add(offering);
						}
					}
				default:
					break;
				}
				eventType = parser.next();
			}
			return new SensorItem(procedure, offeringList.toArray(new Offering[offeringList.size()]), null);
			
		} catch (IllegalStateException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (XmlPullParserException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		return null;
	}
}
		
//		
//		try {
//			Document doc = NetworkUtils.httpResponseToDocument(response);
//			NodeList offeringNodesList = doc.getElementsByTagName("sos:ObservationOffering");
//			Log.d("CapabilitiesRequest", "Offerings found: "+offeringNodesList.getLength());
//			
//			for (int i = 0; i < offeringNodesList.getLength(); i++) {
//				String offering = null;
//				if (offeringNodesList.item(i).getAttributes().getLength() > 0) {
//					offering = offeringNodesList.item(i).getAttributes().item(0).getNodeValue();
//				}
//				String offeringName = null;
//				String observedProperty = null;
//				boolean procedureFound = false;
//				String srsName = null;
//				NodeList offeringItems = offeringNodesList.item(i).getChildNodes();
//				for (int j = 0; j < offeringItems.getLength(); j++) {
//					
//					// check if procedure is listed
//					if (offeringItems.item(j).getNodeName().equalsIgnoreCase("sos:procedure")) {
//						if (offeringItems.item(j).getAttributes().item(0).getNodeValue().equalsIgnoreCase(procedure)) {
//							procedureFound = true;
//						}
//					} 
//					
//					// look for observedProperty
//					else if (offeringItems.item(j).getNodeName().equalsIgnoreCase("sos:observedProperty")) {
//						// FIXME: remove this fix for weatherSOS Tests
//						if (!offeringItems.item(j).getAttributes().item(0).getNodeValue().startsWith("urn:x-ogc")) {
//							observedProperty = offeringItems.item(j).getAttributes().item(0).getNodeValue();
//						}
//					}
//					
//					// look for offeringName
//					else if (offeringItems.item(j).getNodeName().equalsIgnoreCase("gml:name")) {
//						offeringName = offeringItems.item(j).getTextContent();
//					}
//					
//					// look for srsName
//					else if (offeringItems.item(j).getNodeName().equalsIgnoreCase("gml:boundedBy")) {
//						if (offeringItems.item(j).getChildNodes().item(1).hasAttributes()) {
//							srsName = offeringItems.item(j).getChildNodes().item(1).getAttributes().item(0).getNodeValue();
//							Log.i("CapabilitiesRequest", "srsName found: "+ srsName);
////						} else {
////							Log.i("CapabilitiesRequest", "no srsName found. Setting to default: "+ CapabilitiesRequest.SRS_NAME);
////							srsName = CapabilitiesRequest.SRS_NAME;
//						}
//						
//					}
//					
//					// Add offering, if procedure was found
//					if (procedureFound && observedProperty != null && offering != null && offeringName != null && srsName != null) {
//						offeringList.add(new Offering(offering, offeringName, observedProperty, srsName, null, null));
//						break;
//					}
//				}
//			}
//			return new SensorItem(procedure, offeringList.toArray(new Offering[offeringList.size()]), null);
//		} catch (UnsupportedEncodingException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		} catch (IllegalStateException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		} catch (IOException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		} catch (ParserConfigurationException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		} catch (SAXException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//		return null;
//	}
//	
//}
