package de.pereira.aqfinder.util.swe;

import java.io.IOException;
import java.util.Date;
import java.util.LinkedHashMap;

import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import android.util.Log;
import android.util.Xml;
import de.pereira.aqfinder.model.OfferingValues;
import de.pereira.aqfinder.util.DateUtils;
import de.pereira.aqfinder.util.NetworkUtils;

public class ObservationRequest {
	
	private static int CRITICAL_DATASET_SIZE = 500;

	
	public static HttpResponse makeGetObservationRequest(String server, String procedure, String offering, String observedProperty, String srsName, Date start, Date end) throws ClientProtocolException, IOException{
		Log.d("ObservationRequest", "Starting makeGetObservationRequest");
		Log.d("ObservationRequest", "Server: "+server);
		Log.d("ObservationRequest", "Observed proberty: "+observedProperty);
		
		String request = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
				"<GetObservation service=\"SOS\" version=\"1.0.0\"" +
				" xmlns=\"http://www.opengis.net/sos/1.0\"" +
				" xmlns:ows=\"http://www.opengis.net/ows/1.1\"" +
				" xmlns:gml=\"http://www.opengis.net/gml\"" +
				" xmlns:ogc=\"http://www.opengis.net/ogc\"" +
				" xmlns:om=\"http://www.opengis.net/om/1.0\"" +
				" srsName=\""+srsName+"\">" +
				"<offering>"+offering+"</offering>" +
				
				"<eventTime>" +
					"<ogc:TM_During>" +
						"<ogc:PropertyName>om:samplingTime</ogc:PropertyName>" +
						"<gml:TimePeriod>" +
							"<gml:beginPosition>"+ DateUtils.getConformString(start)+"</gml:beginPosition>" +
							"<gml:endPosition>"+ DateUtils.getConformString(end)+"</gml:endPosition>" +
						"</gml:TimePeriod>" +
					"</ogc:TM_During>" +
				"</eventTime>"+

				"<procedure>"+procedure+"</procedure>" +
				"<observedProperty>"+observedProperty+"</observedProperty>" +
				"<responseFormat>text/xml;subtype=&quot;om/1.0.0&quot;</responseFormat>" +
				"<resultModel>om:Observation</resultModel>" +
				"<responseMode>inline</responseMode>" +
				"</GetObservation>";
		
		Log.d("ObservationRequest", request);
		
		HttpResponse response = NetworkUtils.execute(server, request);
		Log.d("ObservationRequest", "getObservation request completed");

		return response; 
	}
	
	
	public static OfferingValues parseGetObservationRequest(HttpResponse response){
		LinkedHashMap<String, String> date_values = new LinkedHashMap<String, String>();
		String uom = "";
		String tokenSeparator = null;
		String blockSeparator = null;
		
		XmlPullParser parser = Xml.newPullParser();
		
		try {
			parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, true);
			parser.setInput(response.getEntity().getContent(), null);
			int eventType = parser.getEventType();
			
			while (eventType != XmlPullParser.END_DOCUMENT) {
				String name = null;
				switch (eventType) {
				case XmlPullParser.START_TAG:
					name = parser.getName();
					
					// amount of data in the dataset
					if (name.equalsIgnoreCase("value")) {
						int datasetSize = Integer.parseInt(parser.nextText());
						if (datasetSize > CRITICAL_DATASET_SIZE) {
							// TODO warn the user
						}
					}else if (name.equalsIgnoreCase("uom")) {
						uom = parser.getAttributeValue(0);
					}else if (name.equalsIgnoreCase("TextBlock")) {
						for (int i = 0; i < parser.getAttributeCount(); i++) {
							if (parser.getAttributeName(i).equalsIgnoreCase("tokenSeparator")) {
								tokenSeparator = parser.getAttributeValue(i);
							}else if (parser.getAttributeName(i).equalsIgnoreCase("blockSeparator")) {
								blockSeparator = parser.getAttributeValue(i);
							}
						}
					}else if (name.equalsIgnoreCase("values")) {
						String valuesInOneString = parser.nextText();
						String[] values = valuesInOneString.split(blockSeparator);
						for (int i = 0; i < values.length; i++) {
							date_values.put(values[i].split(tokenSeparator)[0], values[i].split(tokenSeparator)[2]);
						}
					}
					break;
				default:
					break;
				}
				eventType = parser.next();
			}
			
			return new OfferingValues(date_values, uom);
			
		} catch (XmlPullParserException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (IllegalStateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return new OfferingValues(null, null);
	}
}
