package de.pereira.aqfinder.util.swe;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import android.util.Log;

import com.google.android.maps.GeoPoint;

import de.android1.overlaymanager.ManagedOverlayItem;
import de.pereira.aqfinder.model.SIRInformationObject;
import de.pereira.aqfinder.overlay.SensorManagedOverlayItem;
import de.pereira.aqfinder.util.NetworkUtils;

public class SearchSensorRequest {

	/**
	 * @param server
	 * @param topLeft
	 * @param botRight
	 * @param searchTerm 
	 * @return
	 * @throws ClientProtocolException
	 * @throws IOException
	 */
	public static HttpResponse makeSearchSensorRequest(String server, GeoPoint topLeft, GeoPoint botRight, String searchTerm) throws ClientProtocolException, IOException{
		Log.d("SearchSensorRequest", "Coordinates from GeoPoint: "+topLeft+" and "+botRight);
		Log.d("SearchSensorRequest", "Top-Left: "+(topLeft.getLatitudeE6()*1E-6)+" "+topLeft.getLongitudeE6()*1E-6);
		Log.d("SearchSensorRequest", "Bottom-Right: " + (botRight.getLatitudeE6()*1E-6)+" "+botRight.getLongitudeE6()*1E-6);
		
		Log.d("SearchSensorRequest", "Starting makeSearchSensor request");

		String request = "<SearchSensorRequest service=\"SIR\" sir:version=\"0.3.1\" xsi:schemaLocation=\"http://swsl.uni-muenster.de/sir http://giv-genesis.uni-muenster.de/schemas/sir/sirAll.xsd http://www.opengis.net/sensorML/1.0.1 http://schemas.opengis.net/sensorML/1.0.1/sensorML.xsd\" " +
				"xmlns=\"http://swsl.uni-muenster.de/sir\" xmlns:sir=\"http://swsl.uni-muenster.de/sir\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:ns=\"http://www.opengis.net/ows/1.1\">" +
				"<SearchCriteria>" +
					"<SearchText>"+searchTerm+"</SearchText>" +
					"<BoundingBox>" +
						"<ns:LowerCorner>"+ botRight.getLatitudeE6()*1E-6 +" "+ botRight.getLongitudeE6()*1E-6 +"</ns:LowerCorner>" +
						"<ns:UpperCorner>"+ topLeft.getLatitudeE6()*1E-6 +" "+ topLeft.getLongitudeE6()*1E-6 +"</ns:UpperCorner>" +
					"</BoundingBox>" +
				"</SearchCriteria>" +
				"<SimpleResponse>true</SimpleResponse>" +
				"</SearchSensorRequest>";
		
		Log.d("SearchSensorRequest", "SearchSensorRequest: "+request);
		
		HttpResponse response = NetworkUtils.execute(server, request);
		Log.d("SearchSensorRequest", "Search Sensor Request completed");

		return response; 
	}
	
	
	/**
	 * @param response
	 * @return
	 */
	public static List<? extends ManagedOverlayItem> parseSearchSensorRequestResponse(HttpResponse response){
		try {
			Document doc = NetworkUtils.httpResponseToDocument(response);
			NodeList sensorNodes = doc.getElementsByTagName("sir:SearchResultElement");
			Log.d("SearchSensorRequest", "Sensors found: "+sensorNodes.getLength());
			
			List<ManagedOverlayItem> sensors = new ArrayList<ManagedOverlayItem>();
			
			// add sensors to the list
			for (int i = 0; i < sensorNodes.getLength(); i++) {
				ManagedOverlayItem item = extractSensorFromNode(sensorNodes.item(i));
				if (item != null) {
					sensors.add(item);
				}
			}
			return sensors;
//			return parseXML(getExampleSIRResponse());
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalStateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// in case of error
		return new ArrayList<ManagedOverlayItem>();
	}
	
	
	
	private static ManagedOverlayItem extractSensorFromNode(Node searchResultElement) {
		Double lat = null;
		Double lng = null;
		String description = "no description available";
		String descriptionURL = "no descriptionURL available";
		String serviceType = "no service available";
		String sirID = "no sirID available";
		String sensorID = "no ID available";
		String serviceURL = null;
		if (searchResultElement != null) {
			for (int i = 0; i < searchResultElement.getChildNodes().getLength(); i++) {
				if (searchResultElement.getChildNodes().item(i).getNodeName().equals("sir:SensorIDInSIR")) {
					sirID = searchResultElement.getChildNodes().item(i).getTextContent();
				} else
				if (searchResultElement.getChildNodes().item(i).getNodeName().equals("sir:SimpleSensorDescription")) {
					Node simpleSensorDescriptionNode = searchResultElement.getChildNodes().item(i);
					for (int j = 0; j < simpleSensorDescriptionNode.getChildNodes().getLength(); j++) {
						if (simpleSensorDescriptionNode.getChildNodes().item(j).getNodeName().equals("sir:SensorDescriptionURL")) {
							descriptionURL = simpleSensorDescriptionNode.getChildNodes().item(j).getTextContent();
//							Log.d("SearchSensorRequest", "SensorDescriptionURL:" + simpleSensorDescriptionNode.getChildNodes().item(j).getTextContent());
						} else 
						if (simpleSensorDescriptionNode.getChildNodes().item(j).getNodeName().equals("sir:DescriptionText")) {
							description = stripCDATA(simpleSensorDescriptionNode.getChildNodes().item(j).getTextContent());
						} else 
						if (simpleSensorDescriptionNode.getChildNodes().item(j).getNodeName().equals("sir:ObservedBoundingBox")) {
							Node coords = simpleSensorDescriptionNode.getChildNodes().item(j);
							String[] upperLeftString = null;
							String[] lowerRightString = null;
							for (int c = 0; c < coords.getChildNodes().getLength(); c++) {
								if (coords.getChildNodes().item(c).getNodeName().equals("ows:LowerCorner")) {
									upperLeftString = coords.getChildNodes().item(c).getTextContent().split(" ");
								} else if (coords.getChildNodes().item(c).getNodeName().equals("ows:UpperCorner")) {
									lowerRightString = coords.getChildNodes().item(c).getTextContent().split(" ");
								}
							}
							if (upperLeftString != null && lowerRightString != null) {
								lat = (Double.parseDouble(upperLeftString[0])+Double.parseDouble(lowerRightString[0]))/2;
								lng = (Double.parseDouble(upperLeftString[1])+Double.parseDouble(lowerRightString[1]))/2;
//								Log.d("SearchSensorRequest", "Coordinates: "+lat+", "+lng);
							} else {
								// TODO Fehler schmeißen
								Log.e("SearchSensorRequest", "Error reading coordinates from XML response!");
							}
							
						}
					}
				}else 
				if (searchResultElement.getChildNodes().item(i).getNodeName().equals("sir:ServiceReference")) {
					Node serviceReferenceNode = searchResultElement.getChildNodes().item(i);
					for (int j = 0; j < serviceReferenceNode.getChildNodes().getLength(); j++) {
						if (serviceReferenceNode.getChildNodes().item(j).getNodeName().equals("sir:ServiceType")) {
							serviceType = serviceReferenceNode.getChildNodes().item(j).getTextContent();
						} else 
						if (serviceReferenceNode.getChildNodes().item(j).getNodeName().equals("sir:ServiceSpecificSensorID")) {
							sensorID = serviceReferenceNode.getChildNodes().item(j).getTextContent();
						} else 
						if (serviceReferenceNode.getChildNodes().item(j).getNodeName().equals("sir:ServiceURL")) {
							serviceURL = serviceReferenceNode.getChildNodes().item(j).getTextContent();
//							Log.d("SearchSensorRequest", "serviceURL:" + serviceReferenceNode.getChildNodes().item(j).getTextContent());
						}
					}
				}
			}
			if (lat != null && lng != null && serviceURL != null) {
				SIRInformationObject sirObject = new SIRInformationObject(sensorID, serviceURL, serviceType, sirID, description, descriptionURL);
				return new SensorManagedOverlayItem(new GeoPoint((int) (lat * 1E6), (int) (lng * 1E6)), "Title", "Snippet", sirObject);
			}
		}
		return null;
	}
	
	
	private static String stripCDATA(String s) {
		s = s.trim();
		if (s.startsWith("![CDATA[")) {
			s = s.substring(8);
			
			//FIXME Daniel sagen, dass hier noch eine weiter Klammer zu hingehört
			int i = s.indexOf("]");
			if (i == -1) {
				throw new IllegalStateException(
						"argument starts with <![CDATA[ but cannot find pairing ]]>");
			}
			s = s.substring(0, i);
		}
		return s.trim();
	}
}
